#!/usr/bin/python3

import sys

import getopt
import configparser
import core

TERM_COLORS = {
    "black": "\033[0;30m", "red": "\033[0;31m",
    "green": "\033[0;32m", "brown": "\033[0;33m",
    "blue": "\033[0;34m", "purple": "\033[0;35m",
    "cyan": "\033[0;36m", "light grey": "\033[0;37m",
    "dark grey": "\033[1;30m", "light red": "\033[1;31m",
    "light green": "\033[1;32m", "yellow": "\033[1;33m",
    "light blue": "\033[1;34m", "light purple": "\033[1;35m",
    "light cyan": "\033[1;36m", "white": "\033[1;37m",
    "default": "\033[0m", "reverse": "\033[7m",
    "bold": "\033[1m",
}


def usage():
    print('\n'
          'Usage: cli.py [options] [command]\n'
          '\n'
          '    Simple to-do list manager written in Python\n'
          '\n'
          'Commands:\n'
          '    add <text>       add a task\n'
          '    rm <index>       remove a task\n'
          '    list             list all the tasks\n'
          '    clear            remove all the tasks\n'
          '    do <index>       mark a task as done\n'
          '    undo <index>     mark a task as pending\n'
          '\n'
          'Options:\n'
          '    -h, --help       show the help\n'
          '    -g, --global     use global to-do list\n')


def main(argv):
    try:
        config = configparser.ConfigParser()
        config.read('config.ini')
    except FileNotFoundError:
        print(TERM_COLORS['red'] + 'Config file not found.' + TERM_COLORS['default'])
        sys.exit()
    except PermissionError:
        print(TERM_COLORS['red'] + 'Cannot open file. Permission denied.' + TERM_COLORS['default'])
        sys.exit()

    is_global = False

    try:
        opts, args = getopt.getopt(argv, "hg", ["help", "global"])
    except getopt.GetoptError:
        usage()
        sys.exit(2)

    for opt, arg in opts:
        if opt in ("-h", "--help"):
            usage()
            sys.exit()

        elif opt in ("-g", "--global"):
            is_global = True

    b = core.Blackboard(is_global, config['DEFAULT']['file_name'])

    try:

        if 'add' in args:
            if b.add(args[1]):
                print('New task \"' + args[1] + '\" added.')

        elif 'list' in args or len(args) == 0:  # Default choice if no argument
            f = open(b.file_name, 'r')
            print(f.read())
            f.close()

        elif 'clear' in args:

            i = input('Are you sure? (Y/n)')
            i = i.lower()
            if i in ['yes', 'y', '']:
                b.clear()

        elif 'rm' in args:
            if b.remove(args[1]):
                print('Task successfully removed')

        elif 'do' in args:
            if b.do(args[1]):
                print('Task successfully marked as done')

        elif 'undo' in args:
            if b.undo(args[1]):
                print('Task successfully marked as pending')

        else:
            print('Wrong command. See help (--help or -h).')

    except FileNotFoundError:
        print(TERM_COLORS['red'] + 'File not found.' + TERM_COLORS['default'])
    except PermissionError:
        print(TERM_COLORS['red'] + 'Error: cannot open file. Permission denied.' + TERM_COLORS['default'])
    except (TypeError, IndexError, ValueError):
        print(TERM_COLORS['red'] + 'Error: missing argument or invalid parameters. See help (--help or -h).' +
              TERM_COLORS['default'])

    return 0


if __name__ == '__main__':
    main(sys.argv[1:])
