import os.path


class Blackboard:
    # Constants

    DONE_TASK_PREFIX = '- [x] '
    PENDING_TASK_PREFIX = '- [ ] '

    # Functions

    def __init__(self, is_global, file_name=None):
        """
        Initialize Blackboard
        :param is_global: True if use global file (~/TODO.md). If False, use local file
        :param file_name: Name of the file. Will use "TODO.md" if not specified
        """

        if file_name is None:
            self.file_name = 'TODO.md'
        else:
            self.file_name = file_name

        if is_global:
            self.file_path = os.path.expanduser("~/") + self.file_name
        else:
            self.file_path = self.file_name

    def __contains__(self, item):
        contains = False
        with open(self.file_path, 'r') as readfile:
            for line in readfile:
                if item == str(self._subtract_prefix(line).rstrip()):
                    contains = True
            print(contains)
        return contains

    def __len__(self):
        """
        Return the number of task in the to-do list
        :return:
        """
        length = 0
        with open(self.file_path, 'r') as readfile:
            for line in readfile:
                length += 1
            readfile.close()
        return length - 1  # -1 to remove the title line

    def display(self):
        with open(self.file_path, 'r') as readfile:
            for line in readfile:
                print(line.rstrip())
            readfile.close()
        return True

    def get(self, index):

        if index is None or not str(index).isdigit():
            raise TypeError

        n_line = 0
        data = ''
        with open(self.file_path, 'r') as readfile:
            for line in readfile:
                if int(n_line) == int(index):
                    data = self._subtract_prefix(line).rstrip()
                n_line += 1
            readfile.close()
        return data

    def get_all(self):
        """
        Return the to-do list
        """
        data = []
        with open(self.file_path, 'r') as readfile:
            for line in readfile:
                if line[:1] != '#':
                    data.append(self._subtract_prefix(line).rstrip())
            readfile.close()
        return data

    def clear(self):
        """
        Remove all the tasks
        """

        return self.__rewrite('# ' + self.file_name)

    def add(self, el):
        """
        Add a task at the end of the to-do list
        :param el: the element to add
        """

        if el is None:
            raise TypeError
        if el == '':
            raise ValueError

        with open(self.file_path, 'a') as outfile:

            # If the file is empty, add a title before adding the first task
            if os.stat(self.file_path).st_size == 0:
                print('# ' + self.file_name, file=outfile)

            print(self.PENDING_TASK_PREFIX + str(el), file=outfile)
            outfile.close()

        return True

    def remove(self, index):
        """
        Remove the task with the specified index
        :param index: index of the task to remove
        """

        if index is None:
            raise TypeError
        if index == '':
            raise ValueError

        data = ''
        n_line = 0
        with open(self.file_path, 'r') as readfile:
            for line in readfile:
                if n_line != int(index) and line != '\n':
                    data = ''.join([data, str(line)])  # ''.join is more efficient than '+' to concatenate strings
                n_line += 1
            readfile.close()

        if int(index) > n_line - 1:
            raise ValueError
        else:
            self.__rewrite(data.rstrip())
            return True

    def do(self, index):

        return self.__change_state(self.DONE_TASK_PREFIX, index)

    def undo(self, index):

        return self.__change_state(self.PENDING_TASK_PREFIX, index)

    # Helper Functions

    def __change_state(self, prefix, index):
        """
        Change the status of the task
        :param prefix: PENDING_TASK_PREFIX or DONE_TASK_PREFIX
        :param index: index of the task
        """

        if index is None or not str(index).isdigit():
            raise TypeError
        if index == '':
            raise ValueError

        data = ''
        n_line = 0

        with open(self.file_path, 'r') as readfile:
            for line in readfile:

                if n_line == int(index) and line.rstrip() != '':
                    s_line = self._subtract_prefix(line)
                    data = ''.join(
                        [data, prefix, str(s_line)])  # ''.join is more efficient than '+' to concatenate strings
                else:
                    data += str(line)
                n_line += 1
            readfile.close()

        if int(index) >= n_line:
            raise ValueError
        else:
            self.__rewrite(data)
            return True

    def __rewrite(self, data):
        """
        Rewrite the entire file with the specified data
        :param data: string of data to write
        """

        if data is None:
            raise TypeError

        with open(self.file_path, 'w') as outfile:
            print(data, file=outfile)
            outfile.close()
        return True

    def _subtract_prefix(self, line):
        """
        Subtract line from its prefix
        :param line: the line
        :return: the line without its prefix
        """
        return line[len(self.DONE_TASK_PREFIX):]
